/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * <summary> FileProcessor defines the contract to process a file. One file processor defines an algorithm to
 * produce statistical result for a certain class of files against certain metrics. (For instance count line
 * numbers for a text file.) Multiple FileProcessors are aggregated into FileStatistics, which can run in two modes
 * (process with first matching processor and process with each matching processor). File processors are mapped
 * with an alias. This alias is dynamically provided by the user and the reporting can be grouped against this
 * alias. </summary>
 *
 * <contract-constructor> All implementation should have a constructor with a Property as an argument in order to
 * reflect from configuration file. This requirement can be dropped if it's not necessary to use this way.
 * </contract-constructor>
 *
 * <thread-safety> Implementations are advised to be stateless and thread-safe. But it's not a requirement.
 * </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public interface FileProcessor {

    /**
     * <summary> Get a list of all the supported metrics by this processor. This method is mostly for end user. It
     * does not fulfill any responsibility in the current framework. </summary>
     *
     * @return a list of Metric instances
     */
    public List getSupportedMetrics();

    /**
     * <summary> Get the alias of the processor. An alias is the official name of the processor which is also used
     * to classify the stats result against. This should be non-empty. </summary>
     *
     * @return the alias of the file processor
     */
    public String getAlias();

    /**
     * <summary> Get the file matcher associated with the processor in order to determine whether a file can/should
     * be processed with the processor. </summary>
     *
     * @return a file matcher
     */
    public FileMatcher getFileMatcher();

    /**
     * <summary> Process the file and generate the result. A new result should be generated each time, whose keys
     * are the metrics returned from getSupportedMetrics(). </summary>
     *
     * @return the stats result
     * @param file the file to process
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     * @throws FileProcessingException if processor fails
     */
    public StatsResult process(File file) throws IOException, FileProcessingException;

}
