/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.topcoder.file.statistics.reporting.XmlStatsReporting;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;

/**
 * <summary> This is the main class of the user API. It provides the basic API, which includes maintaining a list
 * of file processors, a reporting handler and controls the way to process the file. Files are processed one by one
 * and the results are aggregated into the reporting. If you want to start a new reporting you should either
 * explicitly set the reporting or get the old reporting to reset it. There are two ways to process the file:
 *
 * <br/> - Process it with the first matching processor.
 *
 * <br/> - Process it with each marching processor.
 *
 * <br/> Refer to ExtendedFileStatistics for advanced usage. </summary>
 *
 * <configuration> This class can be intialized from a configuration namespace which should be preloaded. It will
 * look at the following properties:
 *
 * <br/> - "processors" as a container for file processors, each sub-property should contain a "classname" that
 * specifies the fully qualified class name of a FileProcessor implementation which accepts a Property as
 * constructor argument. This property is required.
 *
 * <br/> - "reporting" which contains a "classname" that specifies the fully qualified class name of a
 * StatsReporting implementation which accepts a Property as constructor argument. This property is optional
 * (XmlStatsReporting is the default value).
 *
 * <br/> - "matchall" which is a boolean flag that indicates whether all matching processors will be used to
 * process the file. It is regarded true when value is either "on", "yes", "true" case-insensitively. This property
 * is optional. </configuration>
 *
 * <thread-safety> This class is not thread-safe in that it provides a number of manipulation methods in a
 * non-synchronized manner. Even if calling from different threads at separated time will mix the result into the
 * same reporting, which may not be the desirable behavior. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class FileStatistics {

    /**
     * <summary> This property contains multiple sub-properties to specify FileProcessor implementations. When the
     * implementation is instantiated the property itself is passed in as constructor argument. </summary>
     */
    public static final String PROCESSORS_PROPERTY_KEY = "processors";

    /**
     * <summary> This property contains sub-properties to specify the StatsReporting implementation. When the
     * implementation is instantiated the property itself is passed in as constructor argument. </summary>
     */
    public static final String REPORTING_PROPERTY_KEY = "reporting";

    /**
     * <summary> This property specifies whether a file is processed by the first matching processor or each of the
     * matching processor. This property is optional (default to false). Its value is considered true if it's
     * "true", "yes" or "on". </summary>
     */
    public static final String MATCH_ALL_PROPERTY_KEY = "matchall";

    /**
     * <summary> This is a sub-property in "processors"' sub-properties or "reporting" to specify the fully
     * qualified class name of the FileProcessor or StatsReporting implementation that owns a constructor with a
     * Property argument. </summary>
     */
    public static final String CLASSNAME_PROPERTY_KEY = "classname";

    /**
     * <summary> The processors aggregated. This is intialized to a hashmap and the reference will never change. A
     * LinkedHashMap is used so as to maintain stable order. </summary>
     */
    private Map processors = new LinkedHashMap();

    /**
     * <summary> The reporting to use. This is mutable. </summary>
     */
    private StatsReporting reporting;

    /**
     * <summary> A flag on whether to process the file with each matching processor. </summary>
     */
    private boolean matchAll = false;

    /**
     * <summary> Default constructor. </summary>
     */
    public FileStatistics() {
        this.reporting = new XmlStatsReporting();
    }

    /**
     * <summary> Constructor to load configuration from given namespace. The namespace should be already loaded. It
     * loads the processors, the reporting as well as the matchall flag. </summary>
     *
     * @param namespace the namespace to load from
     *
     * @throws NullPointerException if namespace is null
     * @throws IllegalArgumentException if namespace is empty after trimmed
     * @throws ConfigurationException if config error occurs
     */
    public FileStatistics(String namespace) throws ConfigurationException {
        CheckHelper.checkString(namespace, "namespace");

        // TODO refactor & test
        ConfigManager cm = ConfigManager.getInstance();

        try {
            // obtain "processors" property
            Property processorsProperty = cm.getPropertyObject(namespace, PROCESSORS_PROPERTY_KEY);
            if (processorsProperty == null) {
                throw new ConfigurationException("Missing property " + PROCESSORS_PROPERTY_KEY);
            }

            // for each children of "processors", create and configure a file processor
            Enumeration names = processorsProperty.propertyNames();
            while (names.hasMoreElements()) {
                String name = (String) names.nextElement();
                Property processor = processorsProperty.getProperty(name);

                // obtain "classname" subproperty
                String className = processor.getValue(CLASSNAME_PROPERTY_KEY);
                if (className == null) {
                    throw new ConfigurationException("Missing property " + PROCESSORS_PROPERTY_KEY + "." + name
                            + "." + CLASSNAME_PROPERTY_KEY);
                }

                // instantiate file processor with reflection
                FileProcessor fileProcessor = (FileProcessor) Class.forName(className).getConstructor(
                        new Class[] {Property.class}).newInstance(new Object[] {processor});

                // add created file processor to the collection
                this.processors.put(fileProcessor.getAlias(), fileProcessor);
            }

            // configure reporting, obtain "reporting" optional property
            Property reportingProperty = cm.getPropertyObject(namespace, REPORTING_PROPERTY_KEY);
            if (reportingProperty == null) {
                this.reporting = new XmlStatsReporting();
            } else {
                // obtain "classname" subproperty
                String className = reportingProperty.getValue(CLASSNAME_PROPERTY_KEY);
                if (className == null) {
                    throw new ConfigurationException("Missing property " + REPORTING_PROPERTY_KEY + "."
                            + CLASSNAME_PROPERTY_KEY);
                }

                // instantiate reporting with reflection
                this.reporting = (StatsReporting) Class.forName(className).getConstructor(
                        new Class[] {Property.class}).newInstance(new Object[] {reportingProperty});
            }

            // obtain matchAll optional property
            String matchAllString = cm.getString(namespace, MATCH_ALL_PROPERTY_KEY);
            if (matchAllString != null) {
                this.matchAll = matchAllString.equalsIgnoreCase("true") || matchAllString.equalsIgnoreCase("yes")
                        || matchAllString.equalsIgnoreCase("on");
            }

        } catch (ConfigurationException e) {
            throw e; // rethrow
        } catch (Exception e) {
            throw new ConfigurationException("Failed to configure FileStatistics", e);
        }
    }

    /**
     * <summary> Add the file processor to the aggregation. If there is a processor with the same alias just
     * overwrite the old one. </summary>
     *
     * @param processor the file processor to add
     *
     * @throws NullPointerException if processor is null
     */
    public void addFileProcessor(FileProcessor processor) {
        CheckHelper.checkNull(processor, "processor");

        // we remove the entry if it exists, so as to put the new processor at the end of the collection
        processors.remove(processor.getAlias());

        processors.put(processor.getAlias(), processor);
    }

    /**
     * <summary> Remove the file processor with given alias. If no such process exists return null. </summary>
     *
     * @return the processor just removed
     * @param alias the alias of the processor
     *
     * @throws NullPointerException if alias is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    public FileProcessor removeFileProcessor(String alias) {
        CheckHelper.checkString(alias, "alias");

        return (FileProcessor) processors.remove(alias);
    }

    /**
     * <summary> Get the file processor with given alias. If no such processor exists return null. </summary>
     *
     * @return the associated file processor
     * @param alias the alias of the processor
     *
     * @throws NullPointerException if alias is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    public FileProcessor getFileProcessor(String alias) {
        CheckHelper.checkString(alias, "alias");

        return (FileProcessor) processors.get(alias);
    }

    /**
     * <summary> Get a list of all the processors aggregated. Should return a copy of the inner collection.
     * </summary>
     *
     * @return a list of all the processors
     */
    public List getAllFileProcessors() {
        return new ArrayList(processors.values());
    }

    /**
     * <summary> Clear all the aggregated processors. </summary>
     */
    public void clearAllFileProcessors() {
        processors.clear();
    }

    /**
     * <summary> Get the reporting instance. Simply return the variable. </summary>
     *
     * @return the stats reporting
     */
    public StatsReporting getReporting() {
        return reporting;
    }

    /**
     * <summary> Set the stats reporting implementation. Once the new instance is set the old results are dropped.
     * </summary>
     *
     * @param reporting the stats reporting implementation
     * @throws NullPointerException if reporting is null
     */
    public void setReporting(StatsReporting reporting) {
        CheckHelper.checkNull(reporting, "reporting");
        this.reporting = reporting;
    }

    /**
     * <summary> Whether to process the file with each matching processor. </summary>
     *
     * @return whether to process the file with each matching processor
     */
    public boolean isMatchAll() {
        return matchAll;
    }

    /**
     * <summary> Set whether to process the file with each matching processor. </summary>
     *
     * @param matchAll whether to process the file with each matching processor
     */
    public void setMatchAll(boolean matchAll) {
        this.matchAll = matchAll;
    }

    /**
     * <summary> Process a single file. </summary>
     *
     * <algorithm> 1) If matchall is on, iterate through all file processors; 2) If matchall is off, iterate until
     * we meet the first matching process; 3) Process the file with the matched processor(s); 4) Aggregate the
     * result into reporting; 5) If no processor matches the file nothing happens. </algorithm>
     *
     * @param file the file to process
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     * @throws FileStatisticsException if processing fails
     */
    public void processFile(File file) throws FileStatisticsException, IOException {
        CheckHelper.checkNull(file, "file");

        // for each of the file processors, if the file matches, process the file and add results to report
        for (Iterator iter = processors.values().iterator(); iter.hasNext();) {
            FileProcessor fileProcessor = (FileProcessor) iter.next();
            if (fileProcessor.getFileMatcher().match(file)) {
                StatsResult result = fileProcessor.process(file);
                reporting.aggregate(fileProcessor, file, result);
                // if matchAll flag is disabled, only the first matching processor is reported
                if (!matchAll) {
                    break;
                }
            }
        }
    }
}
