/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.PrintStream;

import com.topcoder.util.commandline.CommandLineUtility;
import com.topcoder.util.commandline.Switch;
import com.topcoder.util.config.ConfigManager;

/**
 * <summary> The command line entry point of the File Statistics utility. It uses ExtendedFileStatistics to fulfill
 * the task. When argument is invalid help message should be printed. </summary>
 *
 * <arguments> java Main [-c config-file] [-n namespace] [-b base-path] [-r] [-o output-file] path [path2 path3
 * ...]
 *
 * <br/> -c config-file: provides a config file to initialize from. It should be in config XML format and is loaded
 * to the namespace (provided or default).
 *
 * <br/> -n namespace: provides a namespace to initialize from If neither is provided the default namespace is
 * used. Provided namespace should be preloaded.
 *
 * <br/> -b base-path: all path will be relative to it.
 *
 * <br/> -r: whether to recursively traverse sub-directory.
 *
 * <br/> -o output-file: the output filename. If missing output is printed to console. </arguments>
 *
 * <thread-safety> This class is stateless and thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class Main {

    /**
     * <summary> The namespace to used if no namespace is provided. </summary>
     */
    public static final String NAMESPACE = "com.topcoder.file.statistics.Main";

    /**
     * <summary> Private constructor to disallow instantiation. </summary>
     */
    private Main() {
        // empty
    }

    /**
     * <summary> Command line entry. Refer to class level doc for options. The options are parsed by Command Line
     * Utility component. </summary>
     *
     * @param args the command line args
     */
    public static void main(String[] args) {

        // without arguments, only print the usage help
        if (args.length <= 1) {
            printHelp();
            return;
        }

        try {
            // configure command line options
            CommandLineUtility utility = new CommandLineUtility();
            utility.addSwitch(new Switch("c", false, 1, 1, null));
            utility.addSwitch(new Switch("n", false, 1, 1, null));
            utility.addSwitch(new Switch("b", false, 1, 1, null));
            utility.addSwitch(new Switch("r", false, 0, 0, null));
            utility.addSwitch(new Switch("o", false, 1, 1, null));

            // parse arguments into option values
            utility.parse(args);
            String configFile = utility.getSwitch("c").getValue();
            String namespace = utility.getSwitch("n").getValue();
            String basePath = utility.getSwitch("b").getValue();
            String outputFile = utility.getSwitch("o").getValue();
            boolean recursive = utility.getSwitch("r").getValues() != null;
            String[] paths = (String[]) utility.getParameters().toArray(new String[0]);

            doFileProcessing(configFile, namespace, basePath, outputFile, recursive, paths);

        } catch (Exception e) {
            e.printStackTrace();
            printHelp();
        }

    }

    /**
     * <p>Gathers file statistics wth given command-line options.</p>
     *
     * @param configFile String wth name of configuration file, may be null
     * @param namespace String with configuration namespace, may be null
     * @param basePath String with base directory path, may be null
     * @param outputFile String with output file name, may be null
     * @param recursive whether to process subdirectories recursively
     * @param paths String[] array with paths to files or directories to process
     *
     * @throws Exception if there was a problem with processing file statistcs
     */
    private static void doFileProcessing(String configFile, String namespace, String basePath, String outputFile,
            boolean recursive, String[] paths) throws Exception {

        // if namespace was not provided, use default
        if (namespace == null) {
            namespace = NAMESPACE;
        }

        // if present, load config file to the namespace
        if (configFile != null) {
            ConfigManager.getInstance().add(namespace, configFile, ConfigManager.CONFIG_XML_FORMAT);
        }

        try {

            // create file statistics and configure the base path
            ExtendedFileStatistics statistics = new ExtendedFileStatistics(namespace);
            if (basePath != null) {
                statistics.getReporting().reset(new File(basePath));
            }

            // process all file and directory paths
            for (int i = 0; i < paths.length; i++) {
                String path = paths[i];
                File file = new File(path);
                if (file.isDirectory()) {
                    statistics.processDirectory(file, recursive);
                } else {
                    statistics.processFile(file);
                }
            }

            // write report to file or console
            if (outputFile != null) {
                statistics.printReport(new File(outputFile));
            } else {
                statistics.printReport();
            }

        } finally {
            // remove loaded namespace
            if (configFile != null) {
                ConfigManager cm = ConfigManager.getInstance();
                if (cm.existsNamespace(namespace)) {
                    cm.removeNamespace(namespace);
                }
            }
        }
    }

    /**
     * <summary> Print help messages to console. A detailed usage info is written to standard output. </summary>
     */
    private static void printHelp() {
        PrintStream out = System.out;
        out.println("java " + Main.class.getName());
        out.println("\t  [-c config-file] [-n namespace] [-b base-path]"
                + " [-r] [-o output-file] path [path2 path3 ...]");
        out.println("-c config-file: provides a config file to initialize from");
        out.println("It should be in config XML format and is loaded to the namespace (provided or default)");
        out.println("-n namespace: provides a namespace to initialize from");
        out.println("If neither is provided the default namespace is used. "
                + "Provided namespace should be preloaded.");
        out.println("-b base-path: all path will be relative to it.");
        out.println("-r: whether to recursively traverse sub-directory.");
        out.println("-o output-file: the output filename. If missing output is printed to console.");
    }
}
