/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.matcher;

import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.util.config.Property;

/**
 * Helper class for argument validation.
 *
 * @author anonymuous, TCSDEVELOPER
 * @version 1.0
 */
final class CheckHelper {

    /** Private constructor to prevent instantiation. */
    private CheckHelper() {
        // empty
    }

    /**
     * Checks whether the given Object is not null.
     *
     * @param obj the argument to check
     * @param name the name of the argument
     * @throws NullPointerException if the given Object is null
     */
    public static void checkNull(Object obj, String name) {
        if (obj == null) {
            throw new NullPointerException(name + " should not be null.");
        }
    }

    /**
     * Checks whether the given String is not null and not empty.
     *
     * @param str the String to check
     * @param name the name of the argument
     * @throws NullPointerException if the given string is null
     * @throws IllegalArgumentException if the given string is empty
     */
    public static void checkString(String str, String name) {
        checkNull(str, name);
        if (str.trim().length() == 0) {
            throw new IllegalArgumentException(name + " should not be empty.");
        }
    }

    /**
     * Checks whether given list is not empty and contains only String instances.
     *
     * @param list the List to check
     * @param name the name of the list
     * @param allowEmptyStrings whether empty strings are allowed
     *
     * @throws NullPointerException if the given list is null
     * @throws IllegalArgumentException if the given list is empty, or contains non-String instance, or contains
     * empty string and empty strings are not allowed
     */
    public static void checkStringList(List list, String name, boolean allowEmptyStrings) {
        checkNull(list, name);

        // ensure that list is not empty
        if (list.isEmpty()) {
            throw new IllegalArgumentException(name + " list is empty");
        }

        // ensure that list contains only String instances
        for (Iterator iter = list.iterator(); iter.hasNext();) {
            Object element = iter.next();
            if (element == null) {
                throw new IllegalArgumentException(name + " list contains null");
            }
            if (!(element instanceof String)) {
                throw new IllegalArgumentException(name + " list contains a non-String instance");
            }
            if (!allowEmptyStrings && ((String) element).trim().length() == 0) {
                throw new IllegalArgumentException(name + " list contains an empty string");
            }
        }

    }

    /**
     * Checks whether given list is not empty and contains only Pattern instances.
     *
     * @param list the List to check
     * @param name the name of the list
     *
     * @throws NullPointerException if the given list is null
     * @throws IllegalArgumentException if the given list is empty, or contains non-Pattern instance
     */
    public static void checkPatternList(List list, String name) {
        checkNull(list, name);

        // ensure that list is not empty
        if (list.isEmpty()) {
            throw new IllegalArgumentException(name + " list is empty");
        }

        // ensure that list contains only Pattern instances
        for (Iterator iter = list.iterator(); iter.hasNext();) {
            Object element = iter.next();
            if (element == null) {
                throw new IllegalArgumentException(name + " list contains null");
            }
            if (!(element instanceof Pattern)) {
                throw new IllegalArgumentException(name + " list contains a non-Pattern instance");
            }
        }

    }

    /**
     * <p>Obtains property values, and ensure thay are not missing nor empty.</p>
     *
     * @param property Property to get values from
     * @param key the property key to get the values
     * @param allowEmptyStrings whether empty strings are allowed
     *
     * @return String[] array with property value
     *
     * @throws ConfigurationException if the property key is missing or empty, or property values contain empty
     * string when not allowed
     */
    public static String[] getPropertyValues(Property property, String key, boolean allowEmptyStrings)
        throws ConfigurationException {
        String[] values = property.getValues(key);
        if (values == null) {
            throw new ConfigurationException("Configuration property " + key + " is missing ");
        }
        if (values.length == 0) {
            throw new ConfigurationException("Configuration property " + key + " is empty");
        }

        if (!allowEmptyStrings) {
            for (int i = 0; i < values.length; i++) {
                if (values[i].trim().length() == 0) {
                    throw new ConfigurationException("A type value must not be empty (after trimmed)");
                }
            }
        }

        return values;
    }

}
