/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.matcher;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.util.config.Property;

/**
 * <summary> FileNameMatcher concrete implementation that matches one or more file extensions. The extensions are
 * matched in a case-sensitive manner. </summary>
 *
 * <configuration> This class can be constructed from a Property instance (reflection contract). The supported
 * property is "extensions" which is potentially multi-valued. </configuration>
 *
 * <thread-safety> This class is stateless and thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class ExtensionMatcher extends FileNameMatcher {

    /**
     * <summary> Configuration key that contains multiple file extensions to match. Though extensions are saved as
     * they are the matching is case-insensitive. Empty is allowed in order to match a file with no extension.
     * </summary>
     */
    public static final String EXTENSIONS_PROPERTY_KEY = "extensions";

    /**
     * <summary> A set of file extensions (String) to match. This collection is initialized in constructor and is
     * immutable. </summary>
     */
    private final Set extensions;

    /**
     * <summary> Constructor with single extension. Simply creates a singleton set to hold it. The extension can be
     * empty. </summary>
     *
     * @param extension a single file extension
     * @throws NullPointerException if extension is null
     */
    public ExtensionMatcher(String extension) {
        CheckHelper.checkNull(extension, "extension");
        this.extensions = Collections.singleton(extension);
    }

    /**
     * <summary> Constructor with multiple extensions. Simply makes a set out of the list. </summary>
     *
     * @param extensions the extensions to match
     *
     * @throws NullPointerException if extensions is null
     * @throws IllegalArgumentException if extensions is empty or contains non-String instance
     */
    public ExtensionMatcher(List extensions) {
        CheckHelper.checkStringList(extensions, "extensions", true);
        this.extensions = new HashSet(extensions);
    }

    /**
     * <summary> Constructor that loads extensions from a property. It should look for an "extensions" sub-property
     * and extract multiple values. </summary>
     *
     * @param property the property to configure from
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if property does not contain an extensions sub-property or it does not hold
     * any values
     */
    public ExtensionMatcher(Property property) throws ConfigurationException {
        CheckHelper.checkNull(property, "property");

        String[] values = CheckHelper.getPropertyValues(property, EXTENSIONS_PROPERTY_KEY, true);

        this.extensions = new HashSet(Arrays.asList(values));
    }

    /**
     * <summary> Checks whether the filename matches any of the extensions. The extension is extracted after the
     * last dot in the filename. If the filename does not contain a dot the extension is empty. Matching should be
     * case-sensitive. </summary>
     *
     * @return whether the filename matches
     * @param filename the filename to match
     */
    protected boolean match(String filename) {
        int ix = filename.lastIndexOf('.');
        String extension = (ix == -1) ? "" : filename.substring(ix + 1);
        return extensions.contains(extension);
    }
}
