/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.matcher;

import java.io.File;
import java.io.IOException;

import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.FileMatchingException;
import com.topcoder.file.type.FileType;
import com.topcoder.file.type.MagicNumbers;

/**
 * <summary> FileTypeMatcher is a template class that handles file type oriented matching logic. File type is
 * resolved by the Magic Numbers component. </summary>
 *
 * <thread-safety> This class is stateless and thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class FileTypeMatcher implements FileMatcher {

    /** An instance of Magic Numbers is cached in this static field. */
    private static MagicNumbers magicNumbers;

    /**
     * <summary> Default constructor. Does nothing. </summary>
     */
    protected FileTypeMatcher() {
        // empty
    }

    /**
     * <summary> It obtains the file type from the Magic Numbers component and uses the template method to decide
     * whether the file matches. The proper configuration of Magic Numbers is not within the responsibility of this
     * class. Any issue around initialization of Magic Numbers is translated into FileMatchingException. Any
     * IOException during obtaining the file type is propagated. If the file does not have a corresponding file
     * type, this would result in a false. </summary>
     *
     * @return whether the file matches
     * @param file the file to match
     *
     * @throws NullPointerException if file is null
     * @throws FileMatchingException if error occurs during initializing Magic Numbers
     * @throws IOException if io exception occurs when determining file type
     */
    public boolean match(File file) throws FileMatchingException, IOException {
        CheckHelper.checkNull(file, "file");
        FileType type = getMagicNumbers().determineFileType(file);
        if (type == null) {
            return false;
        }
        return match(type);
    }

    /**
     * Obtains MagicNumbers instance. The instance is cached in a static field.
     *
     * @return a MagicNumbers instance
     *
     * @throws FileMatchingException if failed to create MagicNumbers
     */
    private static MagicNumbers getMagicNumbers() throws FileMatchingException {
        if (magicNumbers == null) {
            try {
                magicNumbers = new MagicNumbers();
            } catch (Exception e) {
                throw new FileMatchingException("Failed to initalize Magic Numbers component", e);
            }
        }
        return magicNumbers;
    }

    /**
     * <summary> Template method that matches the file with file type result. </summary>
     *
     * @return whether the file matches
     * @param type the file type to match
     *
     * @throws FileMatchingException if error occurs
     */
    protected abstract boolean match(FileType type) throws FileMatchingException;
}
