/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.matcher;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.type.FileType;
import com.topcoder.util.config.Property;

/**
 * <summary> MimeMatcher concrete implementation that matches file types with one or more mimes (as configured in
 * Magic Numbers). </summary>
 *
 * <configuration> This class can be constructed from a Property instance (reflection contract). The supported
 * property is "mimes" which is potentially multi-valued. </configuration>
 *
 * <thread-safety> This class is stateless and thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class MimeMatcher extends FileTypeMatcher {

    /**
     * <summary> Configuration key that contains multiple file mimes to match. Empty mimes are not allowed even if
     * they are valid in the Magic Numbers component. </summary>
     */
    public static final String MIMES_PROPERTY_KEY = "mimes";

    /**
     * <summary> A set of file mimes (String) to match. This collection is initialized in constructor and is
     * immutable. </summary>
     */
    private final Set mimes;

    /**
     * <summary> Constructor with single file mime. Simply creates a singleton set to hold it. </summary>
     *
     * @param mime the file mime
     *
     * @throws NullPointerException if mime is null
     * @throws IllegalArgumentException if mime is empty
     */
    public MimeMatcher(String mime) {
        CheckHelper.checkString(mime, "mime");
        this.mimes = Collections.singleton(mime);
    }

    /**
     * <summary> Constructor with multiple file mimes. Simply makes a set out of the list. </summary>
     *
     * @param mimes the file mimes to match
     *
     * @throws NullPointerException if mimes is null
     * @throws IllegalArgumentException if mimes is empty or contains non-String instance or contains empty string
     * (after trimmed)
     */
    public MimeMatcher(List mimes) {
        CheckHelper.checkStringList(mimes, "mimes", false);
        this.mimes = new HashSet(mimes);
    }

    /**
     * <summary> Constructor that loads mimes from a property. It should look for an "mimes" sub-property and
     * extract multiple values. </summary>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if property does not hold a mimes sub-property or it does not contain any
     * values or any value is empty after trimmed
     */
    public MimeMatcher(Property property) throws ConfigurationException {
        CheckHelper.checkNull(property, "property");

        String[] values = CheckHelper.getPropertyValues(property, MIMES_PROPERTY_KEY, false);

        this.mimes = new HashSet(Arrays.asList(values));
    }

    /**
     * <summary> Check whether any of the mimes match the type. The matching is case-sensitive. </summary>
     *
     * @return whether the type matches
     * @param type the type to match
     */
    protected boolean match(FileType type) {
        return mimes.contains(type.getMime());
    }

}
