/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.matcher;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.type.FileType;
import com.topcoder.util.config.Property;

/**
 * <summary> FileTypeMatcher concrete implementation that matches file types with one or more type names (as
 * configured in Magic Numbers). </summary>
 *
 * <configuration> This class can be constructed from a Property instance (reflection contract). The supported
 * property is "types" which is potentially multi-valued. </configuration>
 *
 * <thread-safety> This class is stateless and thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class TypeNameMatcher extends FileTypeMatcher {

    /**
     * <summary> Configuration key that contains multiple file type names to match. Empty file names are not
     * allowed even if they are valid in the Magic Numbers component. </summary>
     */
    public static final String TYPES_PROPERTY_KEY = "types";

    /**
     * <summary> A set of file name types (String) to match. This collection is initialized in constructor and is
     * immutable. </summary>
     */
    private final Set types;

    /**
     * <summary> Constructor with single file type name. Simply creates a singleton set to hold it. </summary>
     *
     * @param type the file type name
     * @throws NullPointerException if type is null
     * @throws IllegalArgumentException if type is empty
     */
    public TypeNameMatcher(String type) {
        CheckHelper.checkString(type, "type");
        this.types = Collections.singleton(type);
    }

    /**
     * <summary> Constructor with multiple file type names. Simply makes a set out of the list. </summary>
     *
     * @param types the file type names to match
     *
     * @throws NullPointerException if types is null
     * @throws IllegalArgumentException if types is empty or contains non-String instance or contains empty string
     * (after trimmed)
     */
    public TypeNameMatcher(List types) {
        CheckHelper.checkStringList(types, "types", false);
        this.types = new HashSet(types);
    }

    /**
     * <summary> Constructor that loads type names from a property. It should look for an "types" sub-property and
     * extract multiple values. </summary>
     *
     * @param property the property to configure from
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if property does not hold a types sub-property or it does not contain any
     * values or any value is empty after trimmed
     */
    public TypeNameMatcher(Property property) throws ConfigurationException {
        CheckHelper.checkNull(property, "property");

        String[] values = CheckHelper.getPropertyValues(property, TYPES_PROPERTY_KEY, false);

        this.types = new HashSet(Arrays.asList(values));
    }

    /**
     * <summary> Check whether any of the type names match the type. The matching is case-sensitive. </summary>
     *
     * @return whether the type matches
     * @param type the type to match
     */
    protected boolean match(FileType type) {
        return types.contains(type.getName());
    }
}
