/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.processor.linecounter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Collections;
import java.util.List;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.processor.FileProcessorBase;
import com.topcoder.util.config.Property;

/**
 * <summary> CStyleLineCounter counts lines for C style source files. This implementation can handle C, C++, C# or
 * Java. It only takes braces and comma into account so it would not be very accurate in most cases. Comments and
 * string literals can be handled. It supports a single metric of "Line Count". </summary>
 *
 * <configuration> It delegates configuration to base class without any addition. </configuration>
 *
 * <thread-safety> This class is stateless while its base class is thread-safe (thus it is thread-safe too).
 * </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class CStyleLineCounter extends FileProcessorBase {

    /**
     * <summary> The name of the metric used for this processor. </summary>
     */
    public static final String NAME = "Line Count";

    /**
     * <summary> The display name of the metric used for this processor. </summary>
     */
    public static final String DISPLAY_NAME = "line";

    /**
     * <summary> The metric used by this processor. </summary>
     */
    private static final Metric METRIC = new Metric(NAME, DISPLAY_NAME,
            "Number of statements in a source code file.");

    /**
     * <summary> Propagate to super constructor. </summary>
     *
     * @param alias the alias
     * @param matcher the file matcher
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    public CStyleLineCounter(String alias, FileMatcher matcher) {
        super(alias, matcher);
    }

    /**
     * <summary> Propagate to super constructor. </summary>
     *
     * @param property the property to load config from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    public CStyleLineCounter(Property property) throws ConfigurationException {
        super(property);
    }

    /**
     * <summary> Return a list contains single Metric composed of the NAME, DISPLAY_NAME and a description.
     * </summary>
     *
     * @return a list of metrics the processor supports
     */
    public List getSupportedMetrics() {
        return Collections.singletonList(METRIC);
    }

    /**
     * <summary> Count the lines in c style source code file. </summary>
     *
     * <algorithm> 1) Create a text stream. 2) Create a SimpleStateMachine. 3) Process each line with the state
     * machine. 3) Close the stream. 5) Create a StatsResult and place the line count under the specified metric.
     * </algorithm>
     *
     * @return the line count result
     * @param file the file to process
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     */
    public StatsResult process(File file) throws IOException {
        CheckHelper.checkNull(file, "file");

        // create state machine
        SimpleStateMachine state = new SimpleStateMachine();

        // read all the file line by line and process with the state machine
        BufferedReader reader = new BufferedReader(new FileReader(file));
        try {
            for (;;) {
                String line = reader.readLine();
                if (line == null) {
                    break;
                }
                state.process(line);
            }
        } finally {
            reader.close();
        }

        // return counted number of lines as stats result
        StatsResult result = new StatsResult();
        result.setStat(METRIC, state.getLines());
        return result;
    }

    /**
     * <summary> This is a private static inner class to CStyleLineCounter that implements a state machine that can
     * process the source code. Refer to CS for the state macine description.
     *
     * Notice this is an implementation recommendation. Developer can choose to improve from this start point.
     * Dropping this class is acceptable as long as the implementation of CStyleLineCounter is stateless (and the
     * clarity and/or efficiency aspects are improved). </summary>
     *
     * <thread-safety> This class is stateful (by name :) and not thread-safe. Each instance only serves a single
     * process. </thread-safety>
     *
     * @author WishingBone, TCSDEVELOPER
     * @version 1.0
     */
    private static class SimpleStateMachine {

        /**
         * <summary> Represents the state not in comment or literal. </summary>
         */
        private static final int STATE_START = 1;

        /**
         * <summary> Represents the state after a single slash. (Input on slash would enter line comment state;
         * input on star would enter block state.) </summary>
         */
        private static final int STATE_SLASH = 2;

        /**
         * <summary> Represents the state of a star in a block comment. (Input of slash would enter start state,)
         * </summary>
         */
        private static final int STATE_STAR = 3;

        /**
         * <summary> Represents the state of back slash in single quote literal (next character is escaped).
         * </summary>
         */
        private static final int STATE_SQ_BACKSLASH = 4;

        /**
         * <summary> Represents the state of back slash in double quote literal (next character is escaped).
         * </summary>
         */
        private static final int STATE_DQ_BACKSLASH = 5;

        /**
         * <summary> Represents the state in single quote literal. </summary>
         */
        private static final int STATE_SINGLE_QUOTE = 6;

        /**
         * <summary> Represents the state in double quote literal. </summary>
         */
        private static final int STATE_DOUBLE_QUOTE = 7;

        /**
         * <summary> Represents the state in block comment. </summary>
         */
        private static final int STATE_BLOCK_COMMENT = 8;

        /**
         * <summary> Represents the state in line comment. </summary>
         */
        private static final int STATE_LINE_COMMENT = 9;

        /**
         * <summary> Represents any non-special character. </summary>
         */
        private static final int CHAR_OTHER = 0;

        /**
         * <summary> Represents slash character. </summary>
         */
        private static final int CHAR_SLASH = 1;

        /**
         * <summary> Represents star character. </summary>
         */
        private static final int CHAR_STAR = 2;

        /**
         * <summary> Represents backslash character. </summary>
         */
        private static final int CHAR_BACKSLASH = 3;

        /**
         * <summary> Represents single quote character. </summary>
         */
        private static final int CHAR_SINGLE_QUOTE = 4;

        /**
         * <summary> Represents double quote character. </summary>
         */
        private static final int CHAR_DOUBLE_QUOTE = 5;

        /**
         * <summary> Represents line feed character. </summary>
         */
        private static final int CHAR_LINE_FEED = 6;

        /**
         * <summary> An array that maps characters 0..127 into our character codes. </summary>
         */
        private static final int[] CHAR_CODES = prepareCharCodes();

        /**
         * <summary> The length of CHAR_CODES array. </summary>
         */
        private static final int MAX_CHAR = 128;

        /**
         * <summary> An array that maps state and character code into next state. </summary>
         */
        private static final int[][] TRANSITIONS = prepareTransitions();

        /**
         * <summary> The state of this state machine. Reference the consts. </summary>
         */
        private int state = STATE_START;

        /**
         * <summary> The current line count. Initialized to 0 and increments whenever necessary. </summary>
         */
        private long lines = 0;

        /**
         * <summary> Default constructor. </summary>
         */
        public SimpleStateMachine() {
            // empty
        }

        /**
         * <summary> Creates and returns an array that maps characters 0..127 into our character codes. </summary>
         *
         * @return array that maps characters 0..127 into our character codes
         */
        private static int[] prepareCharCodes() {
            int[] result = new int[MAX_CHAR];
            // most of characters fall into category "other"
            for (int i = 0; i < MAX_CHAR; i++) {
                result[i] = CHAR_OTHER;
            }
            // characters relevant for state machine
            char[] chars = new char[] {'/', '*', '\\', '\'', '"', '\n'};
            for (int i = 0; i < chars.length; i++) {
                result[chars[i]] = 1 + i;
            }
            return result;
        }

        /**
         * <summary> Creates and returns an array that maps state and character code into next state. </summary>
         *
         * @return array that maps state and character code into next state
         */
        private static int[][] prepareTransitions() {
            // create the array
            int noOfStates = 1 + STATE_LINE_COMMENT;
            int noOfChars = 1 + CHAR_LINE_FEED;
            int[][] t = new int[noOfStates][noOfChars];

            // most of characters does not change the state
            for (int j = 0; j < noOfStates; j++) {
                for (int i = 0; i < noOfChars; i++) {
                    t[j][i] = j;
                }
            }

            // some states will change to other ones on most cases
            for (int i = 0; i < noOfChars; i++) {
                t[STATE_SLASH][i] = STATE_START;
                t[STATE_STAR][i] = STATE_BLOCK_COMMENT;
                t[STATE_SQ_BACKSLASH][i] = STATE_SINGLE_QUOTE;
                t[STATE_DQ_BACKSLASH][i] = STATE_DOUBLE_QUOTE;
            }

            // sets the interesting trasnsitions
            t[STATE_START][CHAR_SLASH] = STATE_SLASH;
            t[STATE_START][CHAR_DOUBLE_QUOTE] = STATE_DOUBLE_QUOTE;
            t[STATE_START][CHAR_SINGLE_QUOTE] = STATE_SINGLE_QUOTE;
            t[STATE_SLASH][CHAR_STAR] = STATE_BLOCK_COMMENT;
            t[STATE_SLASH][CHAR_SLASH] = STATE_LINE_COMMENT;
            t[STATE_STAR][CHAR_SLASH] = STATE_START;
            t[STATE_STAR][CHAR_STAR] = STATE_STAR;
            t[STATE_SINGLE_QUOTE][CHAR_BACKSLASH] = STATE_SQ_BACKSLASH;
            t[STATE_SINGLE_QUOTE][CHAR_SINGLE_QUOTE] = STATE_START;
            t[STATE_DOUBLE_QUOTE][CHAR_BACKSLASH] = STATE_DQ_BACKSLASH;
            t[STATE_DOUBLE_QUOTE][CHAR_DOUBLE_QUOTE] = STATE_START;
            t[STATE_LINE_COMMENT][CHAR_LINE_FEED] = STATE_START;
            t[STATE_BLOCK_COMMENT][CHAR_STAR] = STATE_STAR;

            // return the created transitions array
            return t;
        }

        /**
         * <summary> Process a line. It processes each character as well as a line feed. </summary>
         *
         * @param line the line to process
         *
         * @throws NullPointerException if line is null
         */
        public void process(String line) {
            CheckHelper.checkNull(line, "line");
            for (int i = 0; i < line.length(); i++) {
                process(line.charAt(i));
            }
            process('\n');
        }

        /**
         * <summary> Process a character from input. Refer to CS for algorithm. </summary>
         *
         * @param ch the character to process
         */
        private void process(char ch) {
            if (state == STATE_START && (ch == ';' || ch == '{')) {
                lines++;
            }
            // change to the next state
            state = TRANSITIONS[state][CHAR_CODES[ch >= MAX_CHAR ? MAX_CHAR - 1 : ch]];
        }

        /**
         * <summary> Get the line count so far. Simply return the variable. </summary>
         *
         * @return the line count
         */
        public long getLines() {
            return lines;
        }
    }

}
