/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

/**
 * <summary> This is the extended version of FileStatistics which supports some advanced operations including
 * directory processing and reporting printing. The reason to separate this out is that we may not need to rely on
 * the directory traversing algorithm incorporated here (which is recursive). </summary>
 *
 * <configuration> This class shares the same configuration logic as FileStatistics. The constructor simply
 * propagates the namespace to the super class. </configuration>
 *
 * <thread-safety> All the operations are delegated. This class does not have any states and is fully dependant on
 * FileStatistics' threading level. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class ExtendedFileStatistics extends FileStatistics {

    /**
     * <summary> Default constructor. Does nothing. </summary>
     */
    public ExtendedFileStatistics() {
        // empty
    }

    /**
     * <summary> Constructor with namespace. Simply propagate to super constructor. </summary>
     *
     * @param namespace the namespace to load config from
     *
     * @throws NullPointerException if namespace is null
     * @throws IllegalArgumentException if namespace is empty after trimmed
     * @throws ConfigurationException if config error occurs
     */
    public ExtendedFileStatistics(String namespace) throws ConfigurationException {
        super(namespace);
    }

    /**
     * <summary> Process a directory in non-recursive mode. Simply calls processDirectory(directory, false).
     * </summary>
     *
     * @param directory the directory
     *
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if argument is not a valid directory
     * @throws IOException if io operation fails
     * @throws FileStatisticsException if processor fails
     */
    public void processDirectory(File directory) throws IOException, FileStatisticsException {
        processDirectory(directory, false);
    }

    /**
     * <summary> Process a directory in given mode. If it's non-recursive only the files in the directory is
     * processed, otherwise every files in sub-directories are processed. </summary>
     *
     * <p>The processing of each file is independent, and so the order in which the directories and files are
     * traversed is not important, and therefore, unspecified. </p>
     *
     * @param directory the directory
     * @param recursive whether to process recursively
     *
     * @throws NullPointerException if directory is null
     * @throws IllegalArgumentException if argument is not a valid directory
     * @throws IOException if io operation fails
     * @throws FileStatisticsException if processor fails
     */
    public void processDirectory(File directory, boolean recursive) throws IOException, FileStatisticsException {
        CheckHelper.checkNull(directory, "directory");
        if (!directory.isDirectory()) {
            throw new IllegalArgumentException("Not a valid directory: " + directory);
        }

        // process all files (and directories, if recursive) in given directory
        File[] files = directory.listFiles();
        for (int i = 0; i < files.length; i++) {
            File file = files[i];
            if (file.isFile()) {
                processFile(file);
            }
            if (recursive && file.isDirectory()) {
                processDirectory(file, true);
            }
        }
    }

    /**
     * <summary> Print the report to console. </summary>
     *
     * @throws StatsReportingException if rendering fails
     */
    public void printReport() throws StatsReportingException {
        System.out.println(getReporting().generateReport());
    }

    /**
     * <summary> Print the report to given file. </summary>
     *
     * @param file the file to write to
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     * @throws StatsReportingException if rendering fails
     */
    public void printReport(File file) throws IOException, StatsReportingException {
        CheckHelper.checkNull(file, "file");
        String report = getReporting().generateReport();

        // write the report to the file
        FileWriter writer = new FileWriter(file);
        try {
            writer.write(report);
        } finally {
            writer.close();
        }
    }
}
