/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics;

/**
 * <summary> Metric represents a measurement of the stats result. A metric is identified by its name. Metrics with
 * the same name will be aggregated. This class overrides equals() and hashCode() so that it can be used as map
 * keys. </summary>
 *
 * <thread-safety> This class is immutable and thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class Metric {

    /**
     * <summary> The name of the metric. Acts as an identifier. </summary>
     */
    private final String name;

    /**
     * <summary> The display name in the rendered result. If this is used in xml reporting the display name should
     * be well formed element name. </summary>
     */
    private final String displayName;

    /**
     * <summary> A description for the metric. This does not make a significance in this release. </summary>
     */
    private final String description;

    /**
     * <summary> Constructor will all arguments initialized. </summary>
     *
     * @param name the name of the metric
     * @param displayName the display name of the metric
     * @param description the description of the metric
     *
     * @throws NullPointerException if any argument is null
     * @throws IllegalArgumentException if any argument is empty after trimmed
     */
    public Metric(String name, String displayName, String description) {
        CheckHelper.checkString(name, "name");
        CheckHelper.checkString(displayName, "displayName");
        CheckHelper.checkString(description, "description");
        this.name = name;
        this.displayName = displayName;
        this.description = description;
    }

    /**
     * <summary> Get the name of the metric. </summary>
     *
     * @return the name of the metric
     */
    public String getName() {
        return name;
    }

    /**
     * <summary> Get the display name of the metric. </summary>
     *
     * @return the display name of the metric
     */
    public String getDisplayName() {
        return displayName;
    }

    /**
     * <summary> Get the description of the metric. </summary>
     *
     * @return the description of the metric
     */
    public String getDescription() {
        return description;
    }

    /**
     * <summary> Equals contract that is based on name. </summary>
     *
     * @param object the object to check
     * @return true, if object is a Metric instance with equal name.
     */
    public boolean equals(Object object) {
        if (!(object instanceof Metric)) {
            return false;
        }
        return ((Metric) object).name.equals(this.name);
    }

    /**
     * <summary> hashCode() contract that is based on name. </summary>
     *
     * @return the hash code
     */
    public int hashCode() {
        return name.hashCode();
    }

}
