/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.matcher;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.util.config.Property;

/**
 * <summary> FileNameMatcher concrete implementation that matches file names with one or more regular expression.
 * The file name matches a regular expression, if it contains a match, for example, pattern
 * "o" wll match <i>topcoder, top, op, code, coder, o</i> but not: <i>tpcder, O, TOPCODER</i>
 * </summary>
 *
 * <configuration> This class can be constructed from a Property instance (reflection contract). The supported
 * property is "patterns" which is potentially multi-valued. </configuration>
 *
 * <thread-safety> This class is stateless and thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class RegexMatcher extends FileNameMatcher {

    /**
     * <summary> Configuration key that contains multiple regular expressions to match. The regex are compiled when
     * loaded and invalid pattern will result in a ConfigurationException. </summary>
     */
    public static final String PATTERNS_PROPERTY_KEY = "patterns";

    /**
     * <summary> A list of compiled regular expressionss (Pattern) to match. This list is initialized in
     * constructor and is immutable. </summary>
     */
    private final List patterns;

    /**
     * <summary> Constructor with single pattern. Simply creates a list to hold it. </summary>
     *
     * @param pattern the regex pattern
     * @throws NullPointerException if pattern is null
     */
    public RegexMatcher(Pattern pattern) {
        CheckHelper.checkNull(pattern, "pattern");
        this.patterns = Collections.singletonList(pattern);
    }

    /**
     * <summary> Constructor with multiple regular expression patterns. Simply save a clone of the list. </summary>
     *
     * @param patterns the patterns to match
     *
     * @throws NullPointerException if patterns is null
     * @throws IllegalArgumentException if patterns is empty or contains non-Pattern instance
     */
    public RegexMatcher(List patterns) {
        CheckHelper.checkPatternList(patterns, "patterns");
        this.patterns = new ArrayList(patterns);
    }

    /**
     * <summary> Constructor that loads patterns from a property. It should look for a "patterns" sub-property and
     * extract multiple values. Each pattern is compiled when loaded. </summary>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if property does not hold a patterns sub-property or it is empty or it
     * contains invalid regular expresssion
     */
    public RegexMatcher(Property property) throws ConfigurationException {
        CheckHelper.checkNull(property, "property");

        String[] values = CheckHelper.getPropertyValues(property, PATTERNS_PROPERTY_KEY, true);

        // compile all patterns from the property values
        this.patterns = new ArrayList();
        for (int i = 0; i < values.length; i++) {
            String regexp = values[i];
            try {
                this.patterns.add(Pattern.compile(regexp));
            } catch (PatternSyntaxException e) {
                throw new ConfigurationException("Invalid regular expression: " + regexp, e);
            }
        }
    }

    /**
     * <summary> Check whether any of the patterns match the filename. </summary>
     *
     * @return whether the filename matches
     * @param filename the filename to match
     */
    protected boolean match(String filename) {
        for (Iterator iter = patterns.iterator(); iter.hasNext();) {
            Pattern pattern = (Pattern) iter.next();
            if (pattern.matcher(filename).find()) {
                return true;
            }
        }
        return false;
    }

}
