/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.processor.linecounter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import com.topcoder.file.statistics.ArgumentHandler;
import com.topcoder.file.statistics.ArgumentOption;
import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.processor.FileProcessorBase;
import com.topcoder.util.config.Property;

/**
 * <summary> TextLineCounter counts lines for text files. It supports a single metric of "Line Count". </summary>
 *
 * <configuration> It delegates configuration to base class. In addition it has an optional configuration property
 * "ignore_blank_line" which is a boolean flag that indicates whether lines of whatespaces should not be counted.
 * It is regarded true when value is either "on", "yes", "true" case-insensitively. This property is optional,
 * default value is false. </configuration>
 *
 * <thread-safety> This class is stateless while its base class is thread-safe (thus it is thread-safe too).
 * </thread-safety>
 *
 * <changes> Since version 2.0b this class adds an option to whether ignore a blank line (line composed of
 * whitespace only). This option can come from either configuration file or command line. </changes>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 */
public class TextLineCounter extends FileProcessorBase implements ArgumentHandler {

    /**
     * <summary> Configuration property key that holds a boolean flag whether to ignore blank line. This is
     * optional and is evaluated to true if the value is "true"/"yes"/"on" case sensitively. Default value is
     * false. </summary>
     *
     * @since 2.0b
     */
    public static final String IGNORE_BLANK_LINE_PROPERTY_KEY = "ignore_blank_line";

    /**
     * <summary>The command line switch name for ignore blank line flag. </summary>
     *
     * @since 2.0b
     */
    public static final String IGNORE_BLANK_LINE_SWITCH_NAME = "ignore_blank_line";

    /**
     * <summary> The name of the metric used for this processor. </summary>
     */
    public static final String NAME = "Line Count";

    /**
     * <summary> The display name of the metric used for this processor. </summary>
     */
    public static final String DISPLAY_NAME = "line";

    /**
     * <summary> The metric used by this processor. </summary>
     */
    private static final Metric METRIC = new Metric(NAME, DISPLAY_NAME, "Number of lines in a text file.");

    /**
     * <summary> Flag whether to ignore blank line. It's set in constructor or by argument handling and never
     * changes. Default is false. </summary>
     *
     * @since 2.0b
     */
    private boolean ignoreBlankLine;

    /**
     * <summary> Propagate to super constructor. </summary>
     *
     * @param alias the alias
     * @param matcher the file matcher
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    public TextLineCounter(String alias, FileMatcher matcher) {
        super(alias, matcher);
    }

    /**
     * <summary> Propagate the super constructor and set ignoreBlankLine flag. </summary>
     *
     * @param alias the alias
     * @param matcher the file matcher
     * @param ignoreBlankLine the flag to ignore blank line
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    public TextLineCounter(String alias, FileMatcher matcher, boolean ignoreBlankLine) {
        super(alias, matcher);
        this.ignoreBlankLine = ignoreBlankLine;
    }

    /**
     * <summary> Propagate to super constructor. </summary>
     *
     * <changes> Since version 2.0b this adds the logic to load the ignore blank line property. </changes>
     *
     * @param property the property to load config from
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    public TextLineCounter(Property property) throws ConfigurationException {
        super(property);
        String value = property.getValue(IGNORE_BLANK_LINE_PROPERTY_KEY);
        if (value != null && (value.equalsIgnoreCase("yes") || value.equalsIgnoreCase("true") || value.equalsIgnoreCase("on"))) {
            ignoreBlankLine = true;
        }
    }

    /**
     * <summary> Return a list contains single Metric composed of the NAME, DISPLAY_NAME and a description.
     * </summary>
     *
     * @return a list of metrics the processor supports
     */
    public List getSupportedMetrics() {
        return Collections.singletonList(METRIC);
    }

    /**
     * <summary> Count the lines in text file. </summary>
     *
     * <algorithm> 1) Create a text stream. 2) Count the lines until EOF is met. 3) Close the stream. 4) Create a
     * StatsResult and place the line count under the specified metric. </algorithm>
     *
     * <changes> Since version 2.0b this method takes the ignoreBlankLine into consideration. </changes>
     *
     * @return the line count result
     * @param file the file to process
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     */
    public StatsResult process(File file) throws IOException {
        CheckHelper.checkNull(file, "file");

        // read text file line by line and count the lines
        long lines = 0;
        BufferedReader reader = new BufferedReader(new FileReader(file));
        try {
            String line;
            while (((line = reader.readLine())) != null) {
                // blank lines are ignored if configured so
                if (ignoreBlankLine && line.trim().length() == 0) {
                    continue;
                }
                lines++;
            }
        } finally {
            reader.close();
        }

        // return number of lines as a stats result
        StatsResult result = new StatsResult();
        result.setStat(METRIC, lines);
        return result;
    }

    /**
     * <summary> Propose a single ArgumentOption with "ignore_blank_line" as switch name and required as false.
     * </summary>
     *
     * @return a list of ArgumentOption instances
     * @since 2.0b
     */
    public List proposeOptions() {
        return Collections.singletonList(new ArgumentOption(IGNORE_BLANK_LINE_SWITCH_NAME,
            "enables ignoring blank lines", false));
    }

    /**
     * <summary> Receive command line argument from the command line interface. It does not care about the actual
     * value of the switch (which is generally not provided). It only cares about if the switch is on or off.
     * </summary>
     *
     * @param arguments the arguments from command line
     * @exception NullPointerException if arguments is null
     * @exception IllegalArgumentExcpetion if "ignore_blank_line" does not correspond to a String
     * @since 2.0b
     */
    public void handleArguments(Map arguments) {
        CheckHelper.checkNull(arguments, "arguments");
        Object value = arguments.get(IGNORE_BLANK_LINE_SWITCH_NAME);
        if (value == null) {
            ignoreBlankLine = false;
        } else if (!(value instanceof String)) {
            throw new IllegalArgumentException("The key '" + IGNORE_BLANK_LINE_SWITCH_NAME
                + "' does not correspond to a String");
        } else {
            ignoreBlankLine = true;
        }
    }

}
