/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.matcher;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

import junit.framework.TestCase;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatchingException;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.util.config.Property;

/**
 * <p>Unit test case for RegexMatcher class.</p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class RegexMatcherTests extends TestCase {

    /** A pattern used in tests. */
    private static final Pattern PATTERN = Pattern.compile(".*\\.txt");

    /** Another pattern used in tests. */
    private static final Pattern PATTERN_2 = Pattern.compile("foobar");

    /** The matcher instance that is tested. */
    private static final RegexMatcher MATCHER = new RegexMatcher(PATTERN);

    /**
     * Creation test for RegexMatcher(Pattern) constructor with valid argument.
     */
    public void testRegexMatcherPattern1() {
        RegexMatcher matcher = new RegexMatcher(PATTERN);

        assertEquals("the pattern were not propagated correctly", getPatterns(matcher), Arrays
                .asList(new String[] {PATTERN.pattern()}));
    }

    /**
     * Returns "patterns" field value from an RegexMatcher by reflection. The Pattern list is transformed by
     * getting pattern source from Pattern objects before returning.
     *
     * @param matcher RegexMatcher to get the field value of
     * @return the value of "patterns" field of the given matcher
     */
    private static List getPatterns(RegexMatcher matcher) {
        List list = (List) TestHelper.getFieldValue(matcher, RegexMatcher.class, "patterns");
        List result = new ArrayList();
        for (Iterator iter = list.iterator(); iter.hasNext();) {
            Pattern pattern = (Pattern) iter.next();
            result.add(pattern.pattern());
        }
        return result;
    }

    /**
     * Creation test for RegexMatcher(Pattern) with null argument. NullPointerException should be thrown.
     */
    public void testRegexMatcherPattern2() {
        try {
            new RegexMatcher((Pattern) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for RegexMatcher(List) constructor with valid argument.
     */
    public void testRegexMatcherList1() {
        List patternList = Arrays.asList(new Pattern[] {PATTERN, PATTERN_2});

        RegexMatcher matcher = new RegexMatcher(patternList);

        assertEquals("the Regexs were not propagated correctly", getPatterns(matcher), Arrays.asList(new String[] {
            PATTERN.pattern(), PATTERN_2.pattern()}));
    }

    /**
     * Creation test for RegexMatcher(List) with null argument. NullPointerException should be thrown.
     */
    public void testRegexMatcherList2() {
        try {
            new RegexMatcher((List) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for RegexMatcher(List) with invalid list. IllegalArgumentException should be thrown.
     *
     * The following cases are verified: empty list, list contains null, list contains non-Pattern instance.
     */
    public void testRegexMatcherList3() {
        List emptyList = Arrays.asList(new Pattern[] {});
        List containsNullList = Arrays.asList(new Pattern[] {PATTERN, null});
        List containsNonStringList = Arrays.asList(new Object[] {new Object(), ""});
        List[] lists = new List[] {emptyList, containsNullList, containsNonStringList};

        // test for each list
        for (int i = 0; i < lists.length; i++) {
            try {
                new RegexMatcher(lists[i]);
                fail("IllegalArgumentException should be thrown for list: " + lists[i]);
            } catch (IllegalArgumentException e) {
                // ok
            }
        }
    }

    /**
     * Creation test for RegexMatcher(Property) constructor with valid argument.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testRegexMatcherProperty1() throws ConfigurationException {
        // test single-valued "patterns" subproperty
        Property property = new Property("name");
        TestHelper.addSubproperty(property, new Property(RegexMatcher.PATTERNS_PROPERTY_KEY, PATTERN.pattern()));
        RegexMatcher matcher = new RegexMatcher(property);

        assertEquals("the pattern were not propagated correctly", getPatterns(matcher), Arrays
                .asList(new String[] {PATTERN.pattern()}));

        // test multi-valued "patterns" subproperty
        property = new Property("name");
        TestHelper.addSubproperty(property, new Property(RegexMatcher.PATTERNS_PROPERTY_KEY, new String[] {
            PATTERN.pattern(), PATTERN_2.pattern()}));
        matcher = new RegexMatcher(property);

        assertEquals("the Regexs were not propagated correctly", getPatterns(matcher), Arrays.asList(new String[] {
            PATTERN.pattern(), PATTERN_2.pattern()}));
    }

    /**
     * Creation test for RegexMatcher(Property) with null argument. NullPointerException should be thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testRegexMatcherProperty2() throws ConfigurationException {
        try {
            new RegexMatcher((Property) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for RegexMatcher(Property) with invalid property. ConfigurationException should be thrown.
     *
     * The following cases are verified: property without patterns subproperty, empty patterns subproperty, invalid
     * regular expression
     */
    public void testRegexMatcherProperty3() {
        // property without "patterns" subproperty
        try {
            new RegexMatcher(new Property("name"));
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // ok
        }

        // empty patterns subproperty
        Property property = new Property("name");
        TestHelper.addSubproperty(property, new Property(RegexMatcher.PATTERNS_PROPERTY_KEY));
        try {
            new RegexMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // ok
        }

        // invalid regular expression
        property = new Property("name");
        String invalidRegExpString = "[foo";
        TestHelper.addSubproperty(property, new Property(RegexMatcher.PATTERNS_PROPERTY_KEY, invalidRegExpString));
        try {
            new RegexMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // ok
        }

    }

    /**
     * Tests match(File) method with null argument. NullPointerException should be thrown.
     *
     * @throws FileMatchingException to JUnit
     */
    public void testMatchFile1() throws FileMatchingException {
        try {
            MATCHER.match((File) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests match(String) method with null argument. NullPointerException should be thrown.
     */
    public void testMatchString1() {
        try {
            MATCHER.match((String) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests match methods with valid arguments. Filename matching "*.txt" should be matched.
     *
     * @throws FileMatchingException to JUnit
     */
    public void testMatch2() throws FileMatchingException {
        // positives
        String[] names = new String[] {"p.txt", "foo.bar.1.2-3.txt", ".txt", "test_files/a/p.txt",
            "test_files/non-existing.txt",  "a.txt."};
        for (int i = 0; i < names.length; i++) {
            assertTrue("file name should be matched, name: " + names[i], MATCHER.match(names[i]));
            assertTrue("file name should be matched, name: " + names[i], MATCHER.match(new File(names[i])));
        }

        // negatives
        names = new String[] {"", "txt.", "a.tx", "test_files/a", "test_files/non-existing"};
        for (int i = 0; i < names.length; i++) {
            assertFalse("file name should not be matched, name: " + names[i], MATCHER.match(names[i]));
            assertFalse("file name should not be matched, name: " + names[i], MATCHER.match(new File(names[i])));
        }
    }

    /**
     * Tests match methods with valid arguments with matcher with two patterns. Filenames that match any pattern
     * should be matched.
     *
     * @throws FileMatchingException to JUnit
     */
    public void testMatch4() throws FileMatchingException {
        RegexMatcher matcher = new RegexMatcher(Arrays.asList(new Pattern[] {PATTERN, PATTERN_2}));
        // positives
        String[] names = new String[] {"p.txt", "foo.bar.1.2-3.txt", ".txt", "test_files/a/p.txt",
            "test_files/non-existing.txt", "foobar", "topfoobarcoder",  "a.txt."};
        for (int i = 0; i < names.length; i++) {
            assertTrue("file name should be matched, name: " + names[i], matcher.match(names[i]));
            assertTrue("file name should be matched, name: " + names[i], matcher.match(new File(names[i])));
        }

        // negatives
        names = new String[] {"", "txt.", "a.tx", "test_files/a", "test_files/non-existing", "FOOBAR"};
        for (int i = 0; i < names.length; i++) {
            assertFalse("file name should not be matched, name: " + names[i], matcher.match(names[i]));
            assertFalse("file name should not be matched, name: " + names[i], matcher.match(new File(names[i])));
        }
    }

}
