/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.processor;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Main;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.matcher.TypeNameMatcher;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;
import com.topcoder.util.config.Property;

/**
 * <p>Unit test case for FileProcessorBase abstract class.</p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class FileProcessorBaseTests extends TestCase {

    /** An alias string used for tests. */
    private static final String ALIAS = "test alias";

    /** A file matcher used for tests. */
    private static final FileMatcher MATCHER = new AnyFileMatcher();

    /** The FileProcessorBase instance that is tested. */
    private static final FileProcessorBase FILE_PROCESSOR = new MyFileProcessorBase(ALIAS, MATCHER);

    /**
     * Creation test of FileProcessorBase(alias, matcher) with null alias. NullPointerException should be thrown.
     */
    public void testFileProcessorBase1() {
        try {
            new MyFileProcessorBase(null, MATCHER);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of FileProcessorBase(alias, matcher) with null matcher. NullPointerException should be thrown.
     */
    public void testFileProcessorBase2() {
        try {
            new MyFileProcessorBase(ALIAS, null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of FileProcessorBase(alias, matcher) with empty string matcher. IllegalArgumentException
     * should be thrown.
     */
    public void testFileProcessorBase3() {
        try {
            new MyFileProcessorBase(" ", MATCHER);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Creation test of FileProcessorBase(alias, matcher) with valid arguments.
     */
    public void testFileProcessorBase4() {
        MyFileProcessorBase processor = new MyFileProcessorBase(ALIAS, MATCHER);
        assertSame("alias was not properly propagated", processor.getAlias(), ALIAS);
        assertSame("file matcher was not properly propagated", processor.getFileMatcher(), MATCHER);
    }

    /**
     * Creation test of FileProcessorBase(property) with null property. NullPointerException should be thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testFileProcessorBase5() throws ConfigurationException {
        try {
            new MyFileProcessorBase(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for FileProcessorBase with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testFileProcessorBase6a() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance()
                .add(Main.NAMESPACE, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor1");
        try {
            MyFileProcessorBase processor = new MyFileProcessorBase(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Text File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof ExtensionMatcher);
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for FileProcessorBase with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testFileProcessorBase6b() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance()
                .add(Main.NAMESPACE, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor2");
        try {
            MyFileProcessorBase processor = new MyFileProcessorBase(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Source File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof TypeNameMatcher);
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for FileProcessorBase with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testFileProcessorBase6c() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance().add(Main.NAMESPACE, "conf/FileStatistics_minimal2.xml",
                ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor1");
        try {
            MyFileProcessorBase processor = new MyFileProcessorBase(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Text File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof AnyFileMatcher);
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for FileProcessorBase with invalid configuration namespace arguments. ConfigurationException
     * should be thrown.
     *
     * @throws ConfigManagerException to JUnit
     */
    public void testFileProcessorBase7() throws ConfigManagerException {
        String[] cases = new String[] {"bad_matcher_classname", "empty_alias", "empty_matcher_extensions",
            "miss_alias", "miss_matcher_classname"};

        for (int i = 0; i < cases.length; i++) {
            String file = "conf/FileStatistics_" + cases[i] + ".xml";
            ConfigManager.getInstance().add(Main.NAMESPACE, file, ConfigManager.CONFIG_XML_FORMAT);
            Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE,
                    "processors.processor1");

            try {
                new MyFileProcessorBase(property);
                fail("ConfigurationException should be thrown. case: " + cases[i]);
            } catch (ConfigurationException e) {
                // ok
            } finally {
                ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
            }
        }
    }

    /**
     * Tests getAlias() getter method.
     */
    public void testGetAlias() {
        assertSame("alias was not properly propagated", FILE_PROCESSOR.getAlias(), ALIAS);
    }

    /**
     * Tests getFileMachter() getter method.
     */
    public void testGetFlieMatcher() {
        assertSame("file matcher was not properly propagated", FILE_PROCESSOR.getFileMatcher(), MATCHER);
    }

    /** A concrete subclass of FileProcessorBase. */
    private static final class MyFileProcessorBase extends FileProcessorBase {

        /**
         * <summary> Constructor with both alias and matcher. Set them to variables respectively. </summary>
         *
         * @param alias the alias for the processor
         * @param matcher the file matcher
         *
         * @throws NullPointerException if alias or matcher is null
         * @throws IllegalArgumentException if alias is empty after trimmed
         */
        public MyFileProcessorBase(String alias, FileMatcher matcher) {
            super(alias, matcher);
        }

        /**
         * <summary> This constructor loads configuration from the property. The non-empty alias and non-null file
         * matcher are required. Any error will be wrapped in a ConfigurationException. </summary>
         *
         * @param property the property to configure from
         *
         * @throws NullPointerException if property is null
         * @throws ConfigurationException if config error occurs
         */
        public MyFileProcessorBase(Property property) throws ConfigurationException {
            super(property);
        }

        /**
         * <summary> Get a list of all the supported metrics by this processor. This method is mostly for end user.
         * It does not fulfill any responsibility in the current framework. </summary>
         *
         * <p>A stub implementations, always return empty list.</p>
         *
         * @return a list of Metric instances
         */
        public List getSupportedMetrics() {
            return new ArrayList();
        }

        /**
         * <summary> Process the file and generate the result. A new result should be generated each time, whose
         * keys are the metrics returned from getSupportedMetrics(). </summary>
         *
         * <p>A stub implementations, always return empty stats.</p>
         *
         * @return the stats result
         * @param file the file to process
         */
        public StatsResult process(File file) {
            return new StatsResult();
        }

    }

}
