/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.processor.linecounter;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import junit.framework.TestCase;

import com.topcoder.file.statistics.ArgumentOption;
import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Main;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.matcher.TypeNameMatcher;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;
import com.topcoder.util.config.Property;

/**
 * <p>Unit test case for CStyleLineCounter class.</p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 */
public class CStyleLineCounterTests extends TestCase {

    /** An alias string used for tests. */
    private static final String ALIAS = "test alias";

    /** A file matcher used for tests. */
    private static final FileMatcher MATCHER = new AnyFileMatcher();

    /** The CStyleLineCounter instance that is tested. */
    private CStyleLineCounter counter;

    /** Initializes the tested counter instance. */
    protected void setUp() throws Exception {
        counter = new CStyleLineCounter(ALIAS, MATCHER);
    }

    /**
     * Creation test of CStyleLineCounter(alias, matcher) with null alias. NullPointerException should be thrown.
     */
    public void testCStyleLineCounter1() {
        try {
            new CStyleLineCounter(null, MATCHER);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of CStyleLineCounter(alias, matcher) with null matcher. NullPointerException should be thrown.
     */
    public void testCStyleLineCounter2() {
        try {
            new CStyleLineCounter(ALIAS, null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of CStyleLineCounter(alias, matcher) with empty string matcher. IllegalArgumentException
     * should be thrown.
     */
    public void testCStyleLineCounter3() {
        try {
            new CStyleLineCounter(" ", MATCHER);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Creation test of CStyleLineCounter(alias, matcher) with valid arguments.
     */
    public void testCStyleLineCounter4() {
        CStyleLineCounter processor = new CStyleLineCounter(ALIAS, MATCHER);
        assertSame("alias was not properly propagated", processor.getAlias(), ALIAS);
        assertSame("file matcher was not properly propagated", processor.getFileMatcher(), MATCHER);
        assertEquals("the supportAtLiteral field value was not set correctly", getSupportAtLiteral(processor),
            false);
    }

    /**
     * Creation test of CStyleLineCounter(alias, matcher, boolean) with null alias. NullPointerException should be
     * thrown.
     */
    public void testCStyleLineCounter1a() {
        try {
            new CStyleLineCounter(null, MATCHER, true);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of CStyleLineCounter(alias, matcher, boolean) with null matcher. NullPointerException should
     * be thrown.
     */
    public void testCStyleLineCounter2a() {
        try {
            new CStyleLineCounter(ALIAS, null, true);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of CStyleLineCounter(alias, matcher, boolean) with empty string matcher.
     * IllegalArgumentException should be thrown.
     */
    public void testCStyleLineCounter3a() {
        try {
            new CStyleLineCounter(" ", MATCHER, true);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Creation test of CStyleLineCounter(alias, matcher, boolean) with valid arguments.
     */
    public void testCStyleLineCounter4a() {
        CStyleLineCounter processor = new CStyleLineCounter(ALIAS, MATCHER, true);
        assertSame("alias was not properly propagated", processor.getAlias(), ALIAS);
        assertSame("file matcher was not properly propagated", processor.getFileMatcher(), MATCHER);
        assertEquals("the supportAtLiteral field value was not set correctly", getSupportAtLiteral(processor),
            true);
    }

    /**
     * Creation test of CStyleLineCounter(property) with null property. NullPointerException should be thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testCStyleLineCounter5() throws ConfigurationException {
        try {
            new CStyleLineCounter(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for CStyleLineCounter with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testCStyleLineCounter6a() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance()
            .add(Main.NAMESPACE, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor1");
        try {
            CStyleLineCounter processor = new CStyleLineCounter(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Text File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof ExtensionMatcher);
            // since 2.0b
            assertFalse("the supportAtLiteral field value was not set correctly", getSupportAtLiteral(processor));
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for CStyleLineCounter with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testCStyleLineCounter6b() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance()
            .add(Main.NAMESPACE, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor2");
        try {
            CStyleLineCounter processor = new CStyleLineCounter(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Source File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof TypeNameMatcher);
            // since 2.0b
            assertTrue("the supportAtLiteral field value was not set correctly", getSupportAtLiteral(processor));
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for CStyleLineCounter with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testCStyleLineCounter6c() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance().add(Main.NAMESPACE, "conf/FileStatistics_minimal2.xml",
            ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor1");
        try {
            CStyleLineCounter processor = new CStyleLineCounter(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Text File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof AnyFileMatcher);
            // since 2.0b
            assertFalse("the supportAtLiteral field value was not set correctly", getSupportAtLiteral(processor));
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for CStyleLineCounter with invalid configuration namespace arguments. ConfigurationException
     * should be thrown.
     *
     * @throws ConfigManagerException to JUnit
     */
    public void testCStyleLineCounter7() throws ConfigManagerException {
        String[] cases = new String[] {"bad_matcher_classname", "empty_alias", "empty_matcher_extensions",
            "miss_alias", "miss_matcher_classname"};

        for (int i = 0; i < cases.length; i++) {
            String file = "conf/FileStatistics_" + cases[i] + ".xml";
            ConfigManager.getInstance().add(Main.NAMESPACE, file, ConfigManager.CONFIG_XML_FORMAT);
            Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE,
                "processors.processor1");

            try {
                new CStyleLineCounter(property);
                fail("ConfigurationException should be thrown. case: " + cases[i]);
            } catch (ConfigurationException e) {
                // ok
            } finally {
                ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
            }
        }
    }

    /**
     * Tests getAlias() getter method.
     */
    public void testGetAlias() {
        assertSame("alias was not properly propagated", counter.getAlias(), ALIAS);
    }

    /**
     * Tests getFileMachter() getter method.
     */
    public void testGetFlieMatcher() {
        assertSame("file matcher was not properly propagated", counter.getFileMatcher(), MATCHER);
    }

    /**
     * Tests getSupportedMetrics() method. Always singleton list with "line" metric should be returned.
     */
    public void testGetSupportedMetrics() {
        List metrics = counter.getSupportedMetrics();
        assertEquals("incorrect supported metrics list", metrics, Arrays.asList(new Object[] {new Metric(
            CStyleLineCounter.NAME, CStyleLineCounter.DISPLAY_NAME, "some description")}));

        // verify metric name and display name
        Metric metric = (Metric) metrics.get(0);
        assertEquals("incorrect supported metric name", metric.getName(), CStyleLineCounter.NAME);
        assertEquals("incorrect supported metric display name", metric.getDisplayName(),
            CStyleLineCounter.DISPLAY_NAME);
    }

    /**
     * Tests process(File) method with null argument. NullPointerException should be thrown.
     *
     * @throws IOException to JUnit
     */
    public void testProcess1() throws IOException {
        try {
            counter.process(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests process(File) method with non-exisitng file. IOException should be thrown.
     */
    public void testProcess2() {
        try {
            counter.process(new File("test_files/not_exists"));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // ok
        }
    }

    /**
     * Tests process(File) method with a directory file. IOException should be thrown.
     */
    public void testProcess3() {
        try {
            counter.process(new File("test_files/a/"));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // ok
        }
    }

    /**
     * Tests process(File) method with valid files. Number of lines should be returned in stats result.
     *
     * @throws IOException to JUnit
     */
    public void testProcess4() throws IOException {
        checkResult(counter.process(new File("test_files/a/p.txt")), 2);
        checkResult(counter.process(new File("test_files/a/empty.txt")), 0);
        checkResult(counter.process(new File("test_files/b/q.jav")), 83);
        checkResult(counter.process(new File("test_files/c/a.c")), 2);
        checkResult(counter.process(new File("test_files/c/b.c")), 1);
        checkResult(counter.process(new File("test_files/c/c.c")), 4);
        // since 2.0b
        checkResult(counter.process(new File("test_files/c/at_literals.c")), 3);
    }

    /**
     * Tests process(File) method with valid files with supportAtLiterals enalbed. Number of lines should be
     * returned in stats result.
     *
     * @throws IOException to JUnit
     */
    public void testProcess5() throws IOException {
        // make counter with enabled support for 'at' literals
        counter = new CStyleLineCounter(counter.getAlias(), counter.getFileMatcher(), true);

        checkResult(counter.process(new File("test_files/a/p.txt")), 2);
        checkResult(counter.process(new File("test_files/a/empty.txt")), 0);
        checkResult(counter.process(new File("test_files/b/q.jav")), 83);
        checkResult(counter.process(new File("test_files/c/a.c")), 2);
        checkResult(counter.process(new File("test_files/c/b.c")), 1);
        checkResult(counter.process(new File("test_files/c/c.c")), 4);

        // and specifically to test the version 2.0 feature - 'at' literals
        checkResult(counter.process(new File("test_files/c/at_literals.c")), 8);
    }

    /**
     * Verifies that result contains the given number of lines counted.
     *
     * @param result stats to check
     * @param lines expected number of lines
     */
    private void checkResult(StatsResult result, int lines) {
        assertTrue("there should be only one metric in stats result", result.getMetrics().size() == 1);
        assertEquals("incorrect line metric in stats result", result.getStat(new Metric(CStyleLineCounter.NAME,
            "-", "-")), lines);
    }

    /**
     * Verifies the argument options proposed by this reporting.
     */
    public void testProposeOptions() {
        List list = counter.proposeOptions();
        assertEquals("there should be one argument option proposed", list.size(), 1);
        assertTrue("there should be ArgumentOption instance proposed", list.get(0) instanceof ArgumentOption);
        ArgumentOption arg = (ArgumentOption) list.get(0);
        assertEquals("incorrect argument switch name", arg.getSwitchName(), "support_at_literal");
        assertEquals("argument shoud be optional", arg.isRequired(), false);
        assertNull("argument should have no validator", arg.getValidator());
    }

    /**
     * Tests handleArguments() method with null argument. NullPointerException should be thrown.
     */
    public void testHandleArguments1() {
        try {
            counter.handleArguments(null);
            fail("exception should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests handleArguments() method with not a String value under switch name key argument.
     * IllegalArgumentException should be thrown.
     */
    public void testHandleArguments2() {
        try {
            counter.handleArguments(Collections.singletonMap("support_at_literal", new Object()));
            fail("exception should be thrown");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Tests handleArguments() method with existing switch name key argument. The flag should be set.
     */
    public void testHandleArguments3() {
        counter.handleArguments(Collections.singletonMap("support_at_literal", ""));
        assertEquals("the field value was not set correctly", getSupportAtLiteral(counter), true);
    }

    /**
     * Tests handleArguments() method with not existing switch name key argument. The flag should be not set.
     */
    public void testHandleArguments4() {
        counter.handleArguments(Collections.singletonMap("no_support_at_literal", ""));
        assertEquals("the field value was not set correctly", getSupportAtLiteral(counter), false);
    }

    /**
     * Helper method to extract supportAtLiteral field value from given CStyleLineCounter.
     *
     * @param counter TextLineCounter to get the field value from.
     * @return the value of supportAtLiteral field from given counter
     */
    private static boolean getSupportAtLiteral(CStyleLineCounter counter) {
        return ((Boolean) TestHelper.getFieldValue(counter, CStyleLineCounter.class, "supportAtLiteral"))
            .booleanValue();
    }

}
