/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.processor.linecounter;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import junit.framework.TestCase;

import com.topcoder.file.statistics.ArgumentOption;
import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Main;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.matcher.TypeNameMatcher;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;
import com.topcoder.util.config.Property;

/**
 * <p>Unit test case for TextLineCounter class.</p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 */
public class TextLineCounterTests extends TestCase {

    /** An alias string used for tests. */
    private static final String ALIAS = "test alias";

    /** A file matcher used for tests. */
    private static final FileMatcher MATCHER = new AnyFileMatcher();

    /** The TextLineCounter instance that is tested. */
    private TextLineCounter counter;

    /** Initializes the tested counter instance. */
    protected void setUp() throws Exception {
        counter = new TextLineCounter(ALIAS, MATCHER);
    }

    /**
     * Creation test of TextLineCounter(alias, matcher, bool) with null alias. NullPointerException should be
     * thrown.
     */
    public void testTextLineCounter1a() {
        try {
            new TextLineCounter(null, MATCHER, true);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of TextLineCounter(alias, matcher, boolean) with null matcher. NullPointerException should be
     * thrown.
     */
    public void testTextLineCounter2a() {
        try {
            new TextLineCounter(ALIAS, null, true);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of TextLineCounter(alias, matcher, boolean) with empty string matcher.
     * IllegalArgumentException should be thrown.
     */
    public void testTextLineCounter3a() {
        try {
            new TextLineCounter(" ", MATCHER, true);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Creation test of TextLineCounter(alias, matcher, boolean) with valid arguments.
     */
    public void testTextLineCounter4a() {
        TextLineCounter processor = new TextLineCounter(ALIAS, MATCHER, true);
        assertSame("alias was not properly propagated", processor.getAlias(), ALIAS);
        assertSame("file matcher was not properly propagated", processor.getFileMatcher(), MATCHER);
        assertEquals("the ignoreBlankLine field value was not set correctly", getIgnoreBlankLine(processor), true);
    }

    /**
     * Creation test of TextLineCounter(alias, matcher) with null alias. NullPointerException should be thrown.
     */
    public void testTextLineCounter1() {
        try {
            new TextLineCounter(null, MATCHER);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of TextLineCounter(alias, matcher) with null matcher. NullPointerException should be thrown.
     */
    public void testTextLineCounter2() {
        try {
            new TextLineCounter(ALIAS, null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of TextLineCounter(alias, matcher) with empty string matcher. IllegalArgumentException should
     * be thrown.
     */
    public void testTextLineCounter3() {
        try {
            new TextLineCounter(" ", MATCHER);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Creation test of TextLineCounter(alias, matcher) with valid arguments.
     */
    public void testTextLineCounter4() {
        TextLineCounter processor = new TextLineCounter(ALIAS, MATCHER);
        assertSame("alias was not properly propagated", processor.getAlias(), ALIAS);
        assertSame("file matcher was not properly propagated", processor.getFileMatcher(), MATCHER);
        assertEquals("the ignoreBlankLine field value was not set correctly", getIgnoreBlankLine(processor), false);
    }

    /**
     * Creation test of TextLineCounter(property) with null property. NullPointerException should be thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testTextLineCounter5() throws ConfigurationException {
        try {
            new TextLineCounter(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for TextLineCounter with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testTextLineCounter6a() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance()
            .add(Main.NAMESPACE, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor1");
        try {
            TextLineCounter processor = new TextLineCounter(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Text File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof ExtensionMatcher);
            // since 2.0b
            assertEquals("the ignoreBlankLine field value was not set correctly", getIgnoreBlankLine(processor), true);
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for TextLineCounter with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testTextLineCounter6b() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance()
            .add(Main.NAMESPACE, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor2");
        try {
            TextLineCounter processor = new TextLineCounter(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Source File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof TypeNameMatcher);
            // since 2.0b
            assertEquals("the ignoreBlankLine field value was not set correctly", getIgnoreBlankLine(processor), false);
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for TextLineCounter with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testTextLineCounter6c() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance().add(Main.NAMESPACE, "conf/FileStatistics_minimal2.xml",
            ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor1");
        try {
            TextLineCounter processor = new TextLineCounter(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Text File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof AnyFileMatcher);
            // since 2.0b
            assertEquals("the ignoreBlankLine field value was not set correctly", getIgnoreBlankLine(processor), false);
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for TextLineCounter with invalid configuration namespace arguments. ConfigurationException
     * should be thrown.
     *
     * @throws ConfigManagerException to JUnit
     */
    public void testTextLineCounter7() throws ConfigManagerException {
        String[] cases = new String[] {"bad_matcher_classname", "empty_alias", "empty_matcher_extensions",
            "miss_alias", "miss_matcher_classname"};

        for (int i = 0; i < cases.length; i++) {
            String file = "conf/FileStatistics_" + cases[i] + ".xml";
            ConfigManager.getInstance().add(Main.NAMESPACE, file, ConfigManager.CONFIG_XML_FORMAT);
            Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE,
                "processors.processor1");

            try {
                new TextLineCounter(property);
                fail("ConfigurationException should be thrown. case: " + cases[i]);
            } catch (ConfigurationException e) {
                // ok
            } finally {
                ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
            }
        }
    }

    /**
     * Tests getAlias() getter method.
     */
    public void testGetAlias() {
        assertSame("alias was not properly propagated", counter.getAlias(), ALIAS);
    }

    /**
     * Tests getFileMachter() getter method.
     */
    public void testGetFlieMatcher() {
        assertSame("file matcher was not properly propagated", counter.getFileMatcher(), MATCHER);
    }

    /**
     * Tests getSupportedMetrics() method. Always singleton list with "line" metric should be returned.
     */
    public void testGetSupportedMetrics() {
        List metrics = counter.getSupportedMetrics();
        assertEquals("incorrect supported metrics list", metrics, Arrays.asList(new Object[] {new Metric(
            TextLineCounter.NAME, TextLineCounter.DISPLAY_NAME, "some description")}));

        // verify metric name and display name
        Metric metric = (Metric) metrics.get(0);
        assertEquals("incorrect supported metric name", metric.getName(), TextLineCounter.NAME);
        assertEquals("incorrect supported metric display name", metric.getDisplayName(),
            TextLineCounter.DISPLAY_NAME);
    }

    /**
     * Tests process(File) method with null argument. NullPointerException should be thrown.
     *
     * @throws IOException to JUnit
     */
    public void testProcess1() throws IOException {
        try {
            counter.process(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests process(File) method with non-exisitng file. IOException should be thrown.
     */
    public void testProcess2() {
        try {
            counter.process(new File("test_files/not_exists"));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // ok
        }
    }

    /**
     * Tests process(File) method with a directory file. IOException should be thrown.
     */
    public void testProcess3() {
        try {
            counter.process(new File("test_files/a/"));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // ok
        }
    }

    /**
     * Tests process(File) method with valid files. Number of lines should be returned in stats result.
     *
     * @throws IOException to JUnit
     */
    public void testProcess4() throws IOException {
        StatsResult result = counter.process(new File("test_files/a/p.txt"));
        assertEquals("incorrect line metric in stats result", getStatLines(result), 22);

        result = counter.process(new File("test_files/a/empty.txt"));
        assertEquals("incorrect line metric in stats result", getStatLines(result), 0);

        result = counter.process(new File("test_files/b/q.jav"));
        assertEquals("incorrect line metric in stats result", getStatLines(result), 275);
    }

    /**
     * Extract lines number from stats result.
     *
     * @param result the StatsResult to extract line numbers from
     * @return number of lines stored in StatsResult
     */
    private long getStatLines(StatsResult result) {
        assertTrue("there should be only one metric in stats result", result.getMetrics().size() == 1);
        return result.getStat(new Metric(TextLineCounter.NAME, "-", "-"));
    }

    /**
     * Tests process(File) method with valid files when ignoreBlankLines is enabled. Number of lines (not including
     * blank lines) should be returned in stats result.
     *
     * @throws IOException to JUnit
     */
    public void testProcess5() throws IOException {
        // make counter with ignoreBlankLines enabled
        counter = new TextLineCounter(counter.getAlias(), counter.getFileMatcher(), true);

        StatsResult result = counter.process(new File("test_files/a/p.txt"));
        assertEquals("incorrect line metric in stats result", getStatLines(result), 15);

        result = counter.process(new File("test_files/a/empty.txt"));
        assertEquals("incorrect line metric in stats result", getStatLines(result), 0);

        result = counter.process(new File("test_files/b/q.jav"));
        assertEquals("incorrect line metric in stats result", getStatLines(result), 242);
    }

    /**
     * Verifies the argument options proposed by this reporting.
     */
    public void testProposeOptions() {
        List list = counter.proposeOptions();
        assertEquals("there should be one argument option proposed", list.size(), 1);
        assertTrue("there should be ArgumentOption instance proposed", list.get(0) instanceof ArgumentOption);
        ArgumentOption arg = (ArgumentOption) list.get(0);
        assertEquals("incorrect argument switch name", arg.getSwitchName(), "ignore_blank_line");
        assertEquals("argument shoud be optional", arg.isRequired(), false);
        assertNull("argument should have no validator", arg.getValidator());
    }

    /**
     * Tests handleArguments() method with null argument. NullPointerException should be thrown.
     */
    public void testHandleArguments1() {
        try {
            counter.handleArguments(null);
            fail("exception should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests handleArguments() method with not a String value under switch name key argument.
     * IllegalArgumentException should be thrown.
     */
    public void testHandleArguments2() {
        try {
            counter.handleArguments(Collections.singletonMap("ignore_blank_line", new Object()));
            fail("exception should be thrown");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Tests handleArguments() method with existing switch name key argument. The flag should be set.
     */
    public void testHandleArguments3() {
        counter.handleArguments(Collections.singletonMap("ignore_blank_line", ""));
        assertEquals("the field value was not set correctly", getIgnoreBlankLine(counter), true);
    }

    /**
     * Tests handleArguments() method with not existing switch name key argument. The flag should be not set.
     */
    public void testHandleArguments4() {
        counter.handleArguments(Collections.singletonMap("no_ignore_blank_line", ""));
        assertEquals("the field value was not set correctly", getIgnoreBlankLine(counter), false);
    }

    /**
     * Helper method to extract ignoreBlankLine field value from given TextLineCounter.
     *
     * @param counter TextLineCounter to get the field value from.
     * @return the value of ignoreBlankLine field from given counter
     */
    private static boolean getIgnoreBlankLine(TextLineCounter counter) {
        return ((Boolean) TestHelper.getFieldValue(counter, TextLineCounter.class, "ignoreBlankLine"))
            .booleanValue();
    }

}
