/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.reporting;

import java.io.File;
import java.util.Collections;
import java.util.List;

import junit.framework.TestCase;

import com.topcoder.file.statistics.ArgumentOption;
import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.InvalidArgumentException;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.util.config.Property;

/**
 * <p>Unit test case for XmlStatsReporting class.</p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 */
public class XmlStatsReportingTests extends TestCase {

    /** A file processor used for tests. */
    private static final FileProcessor PROCESSOR = new TextLineCounter("test alias", new AnyFileMatcher());

    /** A metric used in tests. */
    private static final Metric METRIC = new Metric("name", "lines", "test description");

    /** A stat value used in tests. */
    private static final int STAT_VALUE = 22;

    /** A stylesheet file name used for tests. */
    private static final File STYLESHEET = new File("test_files/stylesheet/xml_to_html.xsl");

    /** A file name used for tests. */
    private static final String FILE_NAME = new File("test_files/a/p.txt").getPath();

    /** A dir name used for tests. */
    private static final String DIR_NAME = new File("test_files/a").getPath();

    /** The instance of tested class. */
    private XmlStatsReporting reporting;

    /**
     * Sets up the test. Initializes tested instance.
     *
     * @throws StatsReportingException to JUnit
     */
    protected void setUp() throws StatsReportingException {
        reporting = new XmlStatsReporting();

        StatsResult result = new StatsResult();
        result.setStat(METRIC, STAT_VALUE);
        reporting.aggregate(PROCESSOR, new File(FILE_NAME), result);
    }

    /**
     * Creation test for XmlStatsReporting default constructor.
     */
    public void testXmlStatsReporting1() {
        XmlStatsReporting newReporting = new XmlStatsReporting();
        assertNull("new reporting should have no base directory set", newReporting.getBaseDirectory());
        assertTrue("new reporting should have empty stats", newReporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getDirectoryStats().getAllKeys()
            .isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getAliasStats().getAllKeys().isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getOverallStats().getMetrics().isEmpty());
    }

    /**
     * Creation test for XmlStatsReporting(File) constructor with null argument. NullPointerException should be
     * thrown.
     */
    public void testXmlStatsReportingFile1() {
        try {
            new XmlStatsReporting((File) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for XmlStatsReporting(File) constructor with invalid file argument. IllegalArgumentException
     * should be thrown.
     */
    public void testXmlStatsReportingFile2() {
        try {
            new XmlStatsReporting(new File(DIR_NAME));
            fail("IllegalArgumentException should be thrown.");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Creation test for XmlStatsReporting(File) constructor with valid stylesheet argument.
     */
    public void testXmlStatsReportingFile3() {
        XmlStatsReporting newReporting = new XmlStatsReporting(STYLESHEET);

        assertNull("new reporting should have no base directory set", newReporting.getBaseDirectory());
        assertTrue("new reporting should have empty stats", newReporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getDirectoryStats().getAllKeys()
            .isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getAliasStats().getAllKeys().isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getOverallStats().getMetrics().isEmpty());
    }

    /**
     * Creation test for XmlStatsReporting(Property) constructor with null argument. NullPointerException should be
     * thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testXmlStatsReportingProperty1() throws ConfigurationException {
        try {
            new XmlStatsReporting((Property) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for XmlStatsReporting(Property) constructor with valid argument.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testXmlStatsReportingProperty2() throws ConfigurationException {
        Property property = new Property("test name");
        TestHelper.addSubproperty(property, new Property(XmlStatsReporting.STYLESHEET_PROPERTY_KEY, STYLESHEET
            .getPath()));
        XmlStatsReporting newReporting = new XmlStatsReporting(property);

        assertNull("new reporting should have no base directory set", newReporting.getBaseDirectory());
        assertTrue("new reporting should have empty stats", newReporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getDirectoryStats().getAllKeys()
            .isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getAliasStats().getAllKeys().isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getOverallStats().getMetrics().isEmpty());
    }

    /**
     * Verifies the argument options proposed by this reporting.
     */
    public void testProposeOptions() {
        List list = reporting.proposeOptions();
        assertEquals("there should be one argument option proposed", list.size(), 1);
        assertTrue("there should be ArgumentOption instance proposed", list.get(0) instanceof ArgumentOption);
        ArgumentOption arg = (ArgumentOption) list.get(0);
        assertEquals("incorrect argument switch name", arg.getSwitchName(), "stylesheet");
        assertEquals("argument shoud be optional", arg.isRequired(), false);
        assertNull("argument should have no validator", arg.getValidator());
    }

    /**
     * Tests handleArguments() method with null argument. NullPointerException should be thrown.
     *
     * @throws InvalidArgumentException to JUnit
     */
    public void testHandleArguments1() throws InvalidArgumentException {
        try {
            reporting.handleArguments(null);
            fail("exception should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests handleArguments() method with not a String value under 'stylesheet' key argument.
     * IllegalArgumentException should be thrown.
     *
     * @throws InvalidArgumentException to JUnit
     */
    public void testHandleArguments2() throws InvalidArgumentException {
        try {
            reporting.handleArguments(Collections.singletonMap("stylesheet", new Object()));
            fail("exception should be thrown");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Tests handleArguments() method with invalid file under 'stylesheet' key argument. InvalidArgumentException
     * should be thrown.
     */
    public void testHandleArguments3() {
        try {
            reporting.handleArguments(Collections.singletonMap("stylesheet", DIR_NAME));
            fail("exception should be thrown");
        } catch (InvalidArgumentException e) {
            // ok
        }
    }

    /**
     * Tests handleArguments() method with valid file under 'stylesheet' key argument. The stylesheet should be
     * set.
     *
     * @throws InvalidArgumentException to JUnit
     */
    public void testHandleArguments4() throws InvalidArgumentException {
        reporting.handleArguments(Collections.singletonMap("stylesheet", STYLESHEET.getAbsolutePath()));
        File fieldValue = (File) TestHelper.getFieldValue(reporting, XmlStatsReporting.class, "stylesheet");
        assertEquals("the stylesheet field value was not set correctly", fieldValue.getAbsolutePath(), STYLESHEET
            .getAbsolutePath());
    }

    /**
     * Tests handleArguments() method with no value under 'stylesheet' key argument. The stylesheet should be not
     * set.
     *
     * @throws InvalidArgumentException to JUnit
     */
    public void testHandleArguments5() throws InvalidArgumentException {
        reporting.handleArguments(Collections.singletonMap("no_stylesheet", STYLESHEET.getAbsolutePath()));
        File fieldValue = (File) TestHelper.getFieldValue(reporting, XmlStatsReporting.class, "stylesheet");
        assertNull("the stylesheet field value was not set correctly", fieldValue);
    }

    /**
     * Tests getAliasStats() getter method.
     */
    public void testGetAliasStats() {
        StatsCollection aliasStats = reporting.getAliasStats();
        assertEquals("incorrect alias stats", aliasStats.get(PROCESSOR.getAlias()).getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests getFileStats() getter method.
     */
    public void testGetFileStats() {
        StatsCollection fileStats = reporting.getFileStats();
        assertEquals("incorrect file stats", fileStats.get(FILE_NAME).getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests getDirectoryStats() getter method.
     */
    public void testGetDirectoryStats() {
        StatsCollection dirStats = reporting.getDirectoryStats();
        assertEquals("incorrect directory stats", dirStats.get(DIR_NAME).getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests getOverallStats() getter method.
     */
    public void testGetOverallStats() {
        StatsResult overallStats = reporting.getOverallStats();
        assertEquals("incorrect overall stats", overallStats.getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests getBaseDirectory() getter method.
     */
    public void testGetBaseDirectory() {
        File baseDir = reporting.getBaseDirectory();
        assertNull("incorrect base directory", baseDir);
    }

    /**
     * Tests reset(File) method with valid directory. The stats should be cleared, and base directory should be
     * set.
     */
    public void testReset1() {
        File dir = new File(DIR_NAME);
        reporting.reset(dir);
        assertSame("incorrect base directory", dir, reporting.getBaseDirectory());

        assertTrue("reporting should have empty stats", reporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getDirectoryStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getAliasStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getOverallStats().getMetrics().isEmpty());
    }

    /**
     * Tests reset(File) method with null directory. The stats should be cleared, and base directory should be set
     * to null.
     */
    public void testReset2() {
        reporting.reset(null);
        assertNull("incorrect base directory", reporting.getBaseDirectory());

        assertTrue("reporting should have empty stats", reporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getDirectoryStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getAliasStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getOverallStats().getMetrics().isEmpty());
    }

    /**
     * Tests reset(File) method with invalid argument. IllegalArgumentException should be thrown.
     */
    public void testReset3() {
        // not a directory
        File file = new File(FILE_NAME);
        try {
            reporting.reset(file);
            fail("IllegalArgumentException");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Tests aggregate method with valid arguments. Stats should be updated appropriately.
     *
     * @throws StatsReportingException to JUnit
     */
    public void testAggregate1() throws StatsReportingException {
        reporting.reset(null);
        StatsResult result = new StatsResult();
        result.setStat(METRIC, STAT_VALUE);

        reporting.aggregate(PROCESSOR, new File(FILE_NAME), result);

        assertEquals("incorrect alias stats", reporting.getAliasStats().get(PROCESSOR.getAlias()).getStat(METRIC),
            STAT_VALUE);
        assertEquals("incorrect file stats", reporting.getFileStats().get(FILE_NAME).getStat(METRIC), STAT_VALUE);
        assertEquals("incorrect directory stats", reporting.getDirectoryStats().get(DIR_NAME).getStat(METRIC),
            STAT_VALUE);
        assertEquals("incorrect overall stats", reporting.getOverallStats().getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests aggregate method with some null argument. NullPointerException should be thrown.
     *
     * @throws StatsReportingException to JUnit
     */
    public void testAggregate2() throws StatsReportingException {
        try {
            reporting.aggregate(null, new File(FILE_NAME), new StatsResult());
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }

        try {
            reporting.aggregate(PROCESSOR, null, new StatsResult());
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }

        try {
            reporting.aggregate(PROCESSOR, new File(FILE_NAME), null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }

    }

    /**
     * Tests aggregate method with valid arguments with baseDir set. Stats should be updated appropriately, and
     * file name should be stripped off the base dir part.
     *
     * @throws StatsReportingException to JUnit
     */
    public void testAggregate3() throws StatsReportingException {
        reporting.reset(new File(DIR_NAME));
        StatsResult result = new StatsResult();
        result.setStat(METRIC, STAT_VALUE);

        reporting.aggregate(PROCESSOR, new File(FILE_NAME), result);

        assertEquals("incorrect alias stats", reporting.getAliasStats().get(PROCESSOR.getAlias()).getStat(METRIC),
            STAT_VALUE);
        assertEquals("incorrect file stats", reporting.getFileStats().get(new File(FILE_NAME).getName()).getStat(
            METRIC), STAT_VALUE);
        assertTrue("incorrect directory stats", reporting.getDirectoryStats().getAllKeys().isEmpty());
        assertEquals("incorrect overall stats", reporting.getOverallStats().getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests aggregate method with file name outside base directory. StatsReportingException should be thrown.
     */
    public void testAggregate4() {
        reporting.reset(new File(DIR_NAME));
        try {
            reporting.aggregate(PROCESSOR, new File("outside/base/dir"), new StatsResult());
            fail("StatsReportingException should be thrown.");
        } catch (StatsReportingException e) {
            // ok
        }
    }

    /**
     * Tests generateReport() method.
     *
     * @throws StatsReportingException to JUnit
     */
    public void testGenerateReport() throws StatsReportingException {
        // prepare some interesting results
        StatsResult stats = new StatsResult();
        stats.setStat(METRIC, 6);
        stats.setStat(new Metric("metric 2", "bugs", "some description..."), 13);
        reporting.aggregate(PROCESSOR, new File("test_files/b/q.java"), stats);

        String report = reporting.generateReport();
        // reggresion test // TestHelper.writeFile(report, "test_files/xml_stats_report.xml");
        assertEquals("incorrect report", report, TestHelper.readFile("test_files/xml_stats_report.xml"));
    }

}
